﻿#region Copyright and file header
//  ********************************************************
//              Muhimbi Document Converter Service
//
//                 Copyright 2009, Muhimbi Ltd
//                       www.muhimbi.com
//
//                     All rights reserved
//
//            Original Document author: jeroen.ritmeijer
//
//       Filename: TestForm.cs
//          To do: -
//   Known issues: -
//        History: 07/10/09 - JR - Initial version
//
//  ********************************************************
#endregion Copyright and file header

using System;
using System.Collections.Generic;
using System.IO;
using System.ServiceModel;
using System.Threading;
using System.Windows.Forms;
using TestHarness.DocumentConverterService;


namespace TestHarness
{
    public partial class TestForm : Form
    {
        private static int _threadCount = 0;
        object paramMissing = Type.Missing;
        
        public TestForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Carry out the conversion of the specified file using the configuration
        /// settings in the form. Conversion is carried out on a separate thread to
        /// allow multiple conversions to take place at once.
        /// </summary>
        /// <param name="sourceFileName">Name of the file to convert including the full path.</param>
        /// <returns>Reference to the thread the covnersion is running on.</returns>
        private Thread ConvertSingleFile(string sourceFileName)
        {
            ThreadStart ts = new ThreadStart(delegate
                {
                    DocumentConverterServiceClient client = null;

                    try
                    {
                        Interlocked.Increment(ref _threadCount);

                        // ** Open the source file and read the contents into a byte array
                        Debug("Opening file: " + sourceFileName);
                        byte[] sourceFile = File.ReadAllBytes(sourceFileName);
                        Debug("File length: " + sourceFile.Length);

                        client = OpenService(textBoxServiceURL.Text);
                        Debug("Client opened");

                        //** Set the various open options as specified in the form.
                        OpenOptions openOptions = new OpenOptions();
                        openOptions.Password = textBoxPassword.Text;
                        openOptions.OriginalFileName = Path.GetFileName(sourceFileName);
                        openOptions.FileExtension = Path.GetExtension(sourceFileName);
                        openOptions.AllowMacros = (MacroSecurityOption)Enum.Parse(typeof(MacroSecurityOption), comboBoxMacroSecurityOptions.Text, true);
                        openOptions.RefreshContent = checkBoxRefreshContent.Checked;

                        // ** Set the various conversion settings as specified inthe form.
                        ConversionSettings conversionSettings = new ConversionSettings();
                        conversionSettings.Fidelity = (ConversionFidelities)Enum.Parse(typeof(ConversionFidelities), comboBoxFidelity.Text, true);
                        conversionSettings.Format = (OutputFormat)Enum.Parse(typeof(OutputFormat), comboBoxFormat.Text, true);
                        conversionSettings.Quality = (ConversionQuality)Enum.Parse(typeof(ConversionQuality), comboBoxQuality.Text, true);
                        conversionSettings.StartPage = int.Parse(textBoxStartPage.Text);
                        conversionSettings.EndPage = int.Parse(textBoxEndPage.Text);
                        conversionSettings.GenerateBookmarks = (BookmarkGenerationOption)Enum.Parse(typeof(BookmarkGenerationOption), comboBoxBookmarks.Text, true);
                        conversionSettings.PDFProfile = (PDFProfile)Enum.Parse(typeof(PDFProfile), comboBoxPDFProfile.Text, true);
                        conversionSettings.OpenPassword = textBoxPDFOpenPassword.Text;
                        conversionSettings.OwnerPassword = textBoxPDFOwnerPassword.Text;
                        if (checkBoxDisableAnnotations.Checked)
                            conversionSettings.SecurityOptions |= SecurityOptions.DisableAnotations;
                        if (checkBoxDisableAccessibility.Checked)
                            conversionSettings.SecurityOptions |= SecurityOptions.DisableContentAccessibility;
                        if (checkBoxDisableCopyContent.Checked)
                            conversionSettings.SecurityOptions |= SecurityOptions.DisableContentCopy;
                        if (checkBoxDisableDocumentAssembly.Checked)
                            conversionSettings.SecurityOptions |= SecurityOptions.DisableDocumentAssembly;
                        if (checkBoxDisableFormFields.Checked)
                            conversionSettings.SecurityOptions |= SecurityOptions.DisableFormFields;
                        if (checkBoxDisableHighResolutionPrint.Checked)
                            conversionSettings.SecurityOptions |= SecurityOptions.DisableHighResolutionPrint;
                        if (checkBoxDisablePrint.Checked)
                            conversionSettings.SecurityOptions |= SecurityOptions.DisablePrint;
                        conversionSettings.Range = (ConversionRange)Enum.Parse(typeof(ConversionRange), comboBoxRange.Text, true);

                        // ** Carry out the actual conversion
                        byte[] convertedFile = client.Convert(sourceFile, openOptions, conversionSettings);
                        Debug("Converted file length: " + convertedFile.Length);
                        Debug("File Converted");

                        string destinationFileName = Path.GetDirectoryName(sourceFileName) + @"\" + Path.GetFileNameWithoutExtension(sourceFileName) + "." + conversionSettings.Format;
                        Debug("Writing file: " + destinationFileName);

                        // ** Let the webbrowser release the file and give it some time. This is a bit hacky
                        NavigateBrowser("about:blank");
                        Thread.Sleep(1000);

                        // ** Write the PDF file to the local file system.
                        using (FileStream fs = File.Create(destinationFileName))
                        {
                            fs.Write(convertedFile, 0, convertedFile.Length);
                            fs.Close();
                        }

                        // ** Dislay the converted file if this option has been enabled.
                        NavigateBrowser(destinationFileName);
                    }
                    catch (FaultException<WebServiceFaultException> ex)
                    {
                        Debug("FaultException occurred: ExceptionType: " + ex.Detail.ExceptionType.ToString());
                    }
                    catch (Exception ex)
                    {
                        Debug(ex.ToString());
                    }
                    finally
                    {
                        CloseService(client);
                        Interlocked.Decrement(ref _threadCount);
                    }
                });

            Thread t = new Thread(ts);
            t.Start();
            return t;
        }

        /// <summary>
        /// Open the service using the specified address and configure the bindings.
        /// </summary>
        /// <returns>An instance of the Web Service.</returns>
        public static DocumentConverterServiceClient OpenService(string address)
        {
            DocumentConverterServiceClient client = null;

            try
            {
                BasicHttpBinding binding = new BasicHttpBinding();
                // ** Use standard Windows Authentication
                binding.Security.Mode = BasicHttpSecurityMode.TransportCredentialOnly;
                binding.Security.Transport.ClientCredentialType = HttpClientCredentialType.Windows;
                // ** Allow long running conversions
                binding.SendTimeout = TimeSpan.FromMinutes(30);
                binding.ReceiveTimeout = TimeSpan.FromMinutes(30);
                // ** Allow file sizes of 50MB
                binding.MaxReceivedMessageSize = 50*1024*1024;
                binding.ReaderQuotas.MaxArrayLength = 50 * 1024 * 1024;
                binding.ReaderQuotas.MaxStringContentLength = 50 * 1024 * 1024;

                // ** We need to specify an identity (any identity) in order to get it past .net3.5 sp1
                EndpointIdentity epi = EndpointIdentity.CreateUpnIdentity("unknown");
                EndpointAddress epa = new EndpointAddress(new Uri(address), epi);

                client = new DocumentConverterServiceClient(binding, epa);

                client.Open();

                return client;
            }
            catch (Exception)
            {
                CloseService(client);
                throw;
            }
        }

        /// <summary>
        /// Check if the client is open and then close it.
        /// </summary>
        /// <param name="client">The client to close</param>
        public static void CloseService(DocumentConverterServiceClient client)
        {
            if (client != null && client.State == CommunicationState.Opened)
                client.Close();
        }


        /// <summary>
        /// Event handler for converting a single file.
        /// </summary>
        private void buttonConvertFile_Click(object sender, EventArgs e)
        {
            try
            {
                ConvertSingleFile(textBoxFileName.Text);

            }
            catch (Exception ex)
            {
                Debug(ex.ToString());
            }
        }

        /// <summary>
        /// Event handler for batch converting an entire folder.
        /// </summary>
        private void buttonConvertFolder_Click(object sender, EventArgs e)
        {
            // ** Maximum number of parallel conversions (note that the real maximum is configured on the server)
            const int MAX_THREAD_COUNT = 3;
            try
            {
                Thread t = null;
                string[] files = Directory.GetFiles(textBoxFileName.Text);
                foreach (string file in files)
                {
                    if (Path.GetExtension(file).ToLower() != ".pdf")
                    {
                        t = ConvertSingleFile(file);
                        do
                        {
                            Thread.Sleep(200);
                            Application.DoEvents();
                        } while (_threadCount >= MAX_THREAD_COUNT);
                    }
                }
                Debug("All files converted");

            }
            catch (Exception ex)
            {
                Debug(ex.ToString());
            }
        }


        delegate void StringParamDelegate(string param);

        /// <summary>
        /// Log a message to the screen.
        /// </summary>
        /// <param name="s">The message to log.</param>
        private void Debug(String s)
        {
            // ** Make sure we are on the right thread
            if (this.InvokeRequired)
            {
                s = Thread.CurrentThread.ManagedThreadId + " - " + s; 
                Delegate d = new StringParamDelegate(Debug);
                this.Invoke(d, new object[] {s});
                return;
            }
            textBoxDebug.Text += DateTime.Now + " - " + s + "\r\n";
            textBoxDebug.SelectionStart = textBoxDebug.Text.Length;
            textBoxDebug.ScrollToCaret();
            Application.DoEvents();
        }

        /// <summary>
        /// Navigate the borwser control to the specified address. 
        /// Used for displaying PDF and XPS files.
        /// </summary>
        /// <param name="address">The address (file) to navigate to.</param>
        private void NavigateBrowser(string address)
        {
            // ** Make sure we are on the right thread
            if (this.InvokeRequired)
            {
                Delegate d = new StringParamDelegate(NavigateBrowser);
                this.Invoke(d, new object[] { address });
                return;
            }
            if (checkBoxShowConverted.Checked == true)
            {
                webBrowser.Navigate(address);
            }
            labelThreadCount.Text = "In progress: " + _threadCount;
        }

        /// <summary>
        /// Request the server's configuration.
        /// </summary>
        private void buttonRequestConfiguration_Click(object sender, EventArgs e)
        {
            DocumentConverterServiceClient client = OpenService(textBoxServiceURL.Text);
            Debug("Client opened");
            
            Configuration configuration = client.GetConfiguration();
            Debug("Service URL: " + configuration.ConversionServerAddress);

            foreach (ConverterConfiguration c in configuration.Converters)
            {
                Debug("Converter Type: " + c.ConverterType.ToString() + " - Fidelity: " + c.SupportedFidelity + " - " + string.Join(",", c.SupportedFileExtensions));
            }

            client.Close();
        }

        /// <summary>
        /// Run Diagnostic test on the Converter Service.
        /// </summary>
        private void buttonRequestDiagnostics_Click(object sender, EventArgs e)
        {
            DocumentConverterServiceClient client = OpenService(textBoxServiceURL.Text);
            Debug("Client opened");

            List<DiagnosticRequestItem> convertersToDiagnose = new List<DiagnosticRequestItem>();
            DiagnosticRequestItem requestItem = new DiagnosticRequestItem();
            requestItem.ConverterType = ConverterType.WordProcessing;
            requestItem.ConversionFidelity = ConversionFidelities.Full;
            convertersToDiagnose.Add(requestItem);

            requestItem = new DiagnosticRequestItem();
            requestItem.ConverterType = ConverterType.InfoPath;
            requestItem.ConversionFidelity = ConversionFidelities.Full;
            convertersToDiagnose.Add(requestItem);

            requestItem = new DiagnosticRequestItem();
            requestItem.ConverterType = ConverterType.SpreadSheets;
            requestItem.ConversionFidelity = ConversionFidelities.Full;
            convertersToDiagnose.Add(requestItem);

            requestItem = new DiagnosticRequestItem();
            requestItem.ConverterType = ConverterType.Presentations;
            requestItem.ConversionFidelity = ConversionFidelities.Full;
            convertersToDiagnose.Add(requestItem);

            requestItem = new DiagnosticRequestItem();
            requestItem.ConverterType = ConverterType.MicrosoftPublisher;
            requestItem.ConversionFidelity = ConversionFidelities.Full;
            convertersToDiagnose.Add(requestItem);

            Diagnostics diagnostics = client.GetDiagnostics(convertersToDiagnose.ToArray());
            
            foreach (DiagnosticResultItem item in diagnostics.Items)
            {
                Debug("ConverterType: " + item.ConverterType + "- Valid: " + item.Valid + " - ExceptionType: " + item.ExceptionType.ToString());
            }
            
            client.Close();
        }

    }
}
